@@ -3,7 +3,7 @@
 #
 # support for tessellations
 #
-#   $Revision: 1.99 $ $Date: 2021/03/17 05:33:57 $
+#   $Revision: 1.111 $ $Date: 2024/02/04 08:04:51 $
 #
 tess <- function(..., xgrid=NULL, ygrid=NULL, tiles=NULL, image=NULL,
                  window=NULL, marks=NULL, keepempty=FALSE,
@@ -23,7 +23,7 @@ tess <- function(..., xgrid=NULL, ygrid=NULL, tiles=NULL, image=NULL,
     stopifnot(is.numeric(ygrid) && all(diff(ygrid) > 0))
     if(!is.null(window))
       warning("Argument 'window' ignored, because xgrid, grid are given")
-    window <- owin(range(xgrid), range(ygrid), unitname=uname)
+    window <- owinInternalRect(range(xgrid), range(ygrid), unitname=uname)
     ntiles <- (length(xgrid)-1) * (length(ygrid)-1)
     out <- list(type="rect", window=window, xgrid=xgrid, ygrid=ygrid, n=ntiles)
   } else if(istiled) {
@@ -127,7 +127,7 @@ print.tess <- function(x, ..., brief=FALSE) {
          rect={
            if(full) {
              unitinfo <- summary(unitname(win))
-             if(equispaced(x$xgrid) && equispaced(x$ygrid)) 
+             if(evenly.spaced(x$xgrid) && evenly.spaced(x$ygrid)) 
                splat("Tiles are equal rectangles, of dimension",
                      signif(mean(diff(x$xgrid)), 5),
                      "x",
@@ -219,7 +219,7 @@ plot.tess <- local({
              vector = {
                #' vector of values.
                #' validate length of vector
-               check.anyvector(values, ntiles, things="tiles")
+               check.anyvector(values, ntiles, things="tiles", vname="values")
              },
              dataframe = {
                #' data frame or matrix of values.
@@ -444,7 +444,7 @@ tiles <- function(x) {
            ny <- length(yg) - 1
            for(j in rev(seq_len(ny))) {
              for(i in seq_len(nx)) {
-               winij <- owin(xg[c(i,i+1)], yg[c(j,j+1)])
+               winij <- owinInternalRect(xg[c(i,i+1)], yg[c(j,j+1)])
                out <- append(out, list(winij))
              }
            }
@@ -573,18 +573,18 @@ tile.areas <- function(x) {
          rect={
            xg <- x$xgrid
            yg <- x$ygrid
-#           nx <- length(xg) - 1 
-#           ny <- length(yg) - 1
            a <- outer(rev(diff(yg)), diff(xg), "*")
            a <- as.vector(t(a))
            names(a) <- as.vector(t(tilenames(x)))
          },
          tiled={
-           a <- unlist(lapply(x$tiles, area))
+           a <- as.numeric(sapply(x$tiles, area))
+           names(a) <- tilenames(x)
          },
          image={
            z <- x$image
-           a <- table(z$v) * z$xstep * z$ystep
+           a <- as.numeric(table(z$v)) * z$xstep * z$ystep
+           names(a) <- tilenames(x)
          })
   return(a)
 }
@@ -592,21 +592,45 @@ tile.areas <- function(x) {
          
 as.im.tess <- function(X, W=NULL, ...,
                        eps=NULL, dimyx=NULL, xy=NULL,
-                       na.replace=NULL) {
-  # if W is present, it may have to be converted
+                       rule.eps=c("adjust.eps", "grow.frame", "shrink.frame"),
+                       na.replace=NULL, values=NULL) {
+  rule.eps <- match.arg(rule.eps)
+  ## if W is present, it may have to be converted
   if(!is.null(W)) {
     stopifnot(is.owin(W))
-    if(W$type != "mask")
-      W <- as.mask(W, eps=eps, dimyx=dimyx, xy=xy)
-  } 
+    if(W$type != "mask") 
+      W <- as.mask(W, eps=eps, dimyx=dimyx, xy=xy, rule.eps=rule.eps)
+  }
+  if(!is.null(values)) {
+    if(!is.atomic(values))
+      stop("Argument 'values' should contain numeric, logical or factor values",
+           call.=FALSE)
+    if(length(values) != nobjects(X))
+      stop(paste("length(values) =", length(values), "!=",
+                 nobjects(X), "= number of tiles"),
+           call.=FALSE)
+  }
   switch(X$type,
          image={
-           out <- as.im(X$image, W=W, eps=eps, dimyx=dimyx, xy=xy,
-                        na.replace=na.replace)
+           if(is.null(values)) {
+             ## result is factor image
+             out <- as.im(X$image, W=W, eps=eps, dimyx=dimyx, xy=xy,
+                          rule.eps=rule.eps,
+                          na.replace=na.replace)
+           } else {
+             ## map tiles to 'values'
+             out <- as.im(X$image, W=W, eps=eps, dimyx=dimyx, xy=xy,
+                          rule.eps=rule.eps)
+             out <- eval.im(values[as.integer(out)])
+             ## replace NA by other value
+             if(!is.null(na.replace))
+               out <- as.im(out, na.replace=na.replace)
+           }
          },
          tiled={
            if(is.null(W))
-             W <- as.mask(as.owin(X), eps=eps, dimyx=dimyx, xy=xy)
+             W <- as.mask(as.owin(X), eps=eps, dimyx=dimyx, xy=xy,
+                          rule.eps=rule.eps)
            til <- X$tiles
            ntil <- length(til)
            nama <- names(til)
@@ -618,32 +642,46 @@ as.im.tess <- function(X, W=NULL, ...,
              tag <- as.im(indic, value=i)
              if(i == 1) {
                out <- tag
-               outv <- out$v
+               outv <- as.integer(out$v)
              } else {
                outv <- pmin.int(outv, tag$v, na.rm=TRUE)
              }
            }
-           out <- im(factor(outv, levels=seq_len(ntil), labels=nama),
-                     out$xcol, out$yrow)
-           unitname(out) <- unitname(W)
+           if(is.null(values)) {
+             ## result is factor image
+             outv <- factor(outv, levels=seq_len(ntil), labels=nama)
+           } else {
+             ## map tiles to 'values'
+             outv <- values[outv]
+           }
+           if(!is.null(na.replace) && anyNA(outv))
+             outv[is.na(outv)] <- na.replace
+           out <- im(outv, out$xcol, out$yrow, unitname=unitname(W))
          },
          rect={
-           if(is.null(W))
-             out <- as.im(as.rectangle(X), eps=eps, dimyx=dimyx, xy=xy)
-           else
-             out <- as.im(W)
+           out <- as.im(W %orifnull% as.rectangle(X),
+                        eps=eps, dimyx=dimyx, xy=xy,
+                        rule.eps=rule.eps)
            xg <- X$xgrid
            yg <- X$ygrid
-           nrows <- length(yg) - 1
-           ncols <- length(xg) - 1
+           nrows <- length(yg) - 1L
+           ncols <- length(xg) - 1L
            jx <- findInterval(out$xcol, xg, rightmost.closed=TRUE)
            iy <- findInterval(out$yrow, yg, rightmost.closed=TRUE)
            M <- as.matrix(out)
            Jcol <- jx[col(M)]
-           Irow <- nrows - iy[row(M)] + 1
-           Ktile <- Jcol + ncols * (Irow - 1)
-           Ktile <- factor(Ktile, levels=seq_len(nrows * ncols))
-           out <- im(Ktile, xcol=out$xcol, yrow=out$yrow,
+           Irow <- nrows - iy[row(M)] + 1L
+           Ktile <- as.integer(Jcol + ncols * (Irow - 1L))
+           if(is.null(values)) {
+             ## result is factor image
+             outv <- factor(Ktile, levels=seq_len(nrows * ncols))
+           } else {
+             ## map tiles to 'values'
+             outv <- values[Ktile]
+           }
+           if(!is.null(na.replace) && anyNA(outv))
+             outv[is.na(outv)] <- na.replace
+           out <- im(outv, xcol=out$xcol, yrow=out$yrow,
                      unitname=unitname(W))
          }
          )
@@ -662,6 +700,7 @@ as.function.tess <- function(x, ..., values=NULL) {
   if(is.null(values)) {
     f <- function(x,y) { tileindex(x,y,V) }
   } else {
+    if(is.data.frame(values)) values <- unlist(values)
     if(length(values) != nobjects(x))
       stop("Length of 'values' should equal the number of tiles", call.=FALSE)
     f <- function(x,y) { values[as.integer(tileindex(x,y,V))] }
@@ -759,7 +798,7 @@ as.tess.owin <- function(X) {
 
 domain.tess <- Window.tess <- function(X, ...) { as.owin(X) } 
 
-intersect.tess <- function(X, Y, ..., keepmarks=FALSE, sep="x") {
+intersect.tess <- function(X, Y, ..., keepempty=FALSE, keepmarks=FALSE, sep="x") {
   X <- as.tess(X)
   check.1.string(sep)
 
@@ -767,34 +806,38 @@ intersect.tess <- function(X, Y, ..., keepmarks=FALSE, sep="x") {
     ## intersection of a tessellation with a window
     if(Y$type == "mask") {
       ## convert to pixel image 
-      result <- as.im(Y)
       Xtiles <- tiles(X)
       seqXtiles <- seq_along(Xtiles)
+      result <- as.im(Y, value=factor(1, levels=seqXtiles))
       for(i in seqXtiles) {
         tilei <- Xtiles[[i]]
         result[tilei] <- i
       }
       result <- result[Y, drop=FALSE]
-      out <- tess(image=result, window=Y)
+      out <- tess(image=result, window=Y, keepempty=keepempty)
       if(keepmarks && !is.null(marx <- marks(X))) {
-        #' identify non-empty tiles
-        tab <- table(factor(result[], levels=seqXtiles))
-        marks(out) <- marksubset(marx, tab > 0)
+        if(keepempty) {
+          marks(out) <- marx
+        } else {
+          #' identify non-empty tiles
+          tab <- table(factor(result[], levels=seqXtiles))
+          marks(out) <- marksubset(marx, tab > 0)
+        }
       }
       return(out)
     } else {
       ## efficient code when Y is a window, retaining names of tiles of X
       Ztiles <- lapply(tiles(X), intersect.owin, B=Y, ..., fatal=FALSE)
-      isempty <- sapply(Ztiles, is.empty)
+      isempty <- !keepempty & sapply(Ztiles, is.empty)
       Ztiles <- Ztiles[!isempty]
       Xwin <- as.owin(X)
       Ywin <- Y
       Zwin <- intersect.owin(Xwin, Ywin)
-      out <- tess(tiles=Ztiles, window=Zwin)
+      out <- tess(tiles=Ztiles, window=Zwin, keepempty=keepempty)
       if(keepmarks) {
         marx <- marks(X)
         if(!is.null(marx))
-          marx <- as.data.frame(marx)[!isempty, ]
+          marx <- as.data.frame(marx)[!isempty, , drop=FALSE]
         marks(out) <- marx
       }
       return(out)
@@ -836,7 +879,7 @@ intersect.tess <- function(X, Y, ..., keepmarks=FALSE, sep="x") {
   for(i in seq_along(Xtiles)) {
     Xi <- Xtiles[[i]]
     Ti <- lapply(Ytiles, intersect.owin, B=Xi, ..., fatal=FALSE)
-    isempty <- sapply(Ti, is.empty)
+    isempty <- !keepempty & sapply(Ti, is.empty)
     nonempty <- !isempty
     if(any(nonempty)) {
       Ti <- Ti[nonempty]
@@ -861,7 +904,7 @@ intersect.tess <- function(X, Y, ..., keepmarks=FALSE, sep="x") {
   Xwin <- as.owin(X)
   Ywin <- as.owin(Y)
   Zwin <- intersect.owin(Xwin, Ywin)
-  out <- tess(tiles=Ztiles, window=Zwin)
+  out <- tess(tiles=Ztiles, window=Zwin, keepempty=keepempty)
   if(keepmarks) 
     marks(out) <- marx
   return(out)
